/*******************************************************
 * TIC TAC TOE Game                                    *
 * by Parthipan Siva                                   *
 *                                                     *
 * Created as demonstration of OOP for                 *
 * SD121 course.                                       *
 *                                                     *
 *  Nov 2005   created                 - Parthipan     *
 *  Nov 2006   added more comments     - Parthipan     *
 *             fixed checkBlockIn2                     *
 *                                                     *
 *******************************************************/
 
 /*
 This is a ComputerPlayer class for a TicTacToe game. It
 is capable of placing Xs or Os given the current configuration
 of the TicTacToe board. The algorithm's primary objective
 is to block the player's win as a result it will not play
 any 2 move win senarios. It will however block 2 move win
 senarios.
 
 This class requires the use of the GameBoard, Player.
 */

#ifndef COMPUTERPLAYER_H
#define COMPUTERPLAYER_H

#include "Player.h"
#include "GameBoard.h"

class ComputerPlayer : public Player
{
public:

    // Constructer
    ComputerPlayer();
    
    // Place a piece
    // INPUT: currentBoard - is the game board on which to place a piece
    void playAMove(GameBoard& currentBoard);

private:
    
    // Temp game board
    int board[3][3];
    
    // Possible configurations to check for
    int winIn2[3][3][6];
    
    // If one of the above checks passes, this contains the loc
    // where the piece is to be placed
    int locForMove[6][2];
    
    // This checks if there are 2 pieces in a row. If this is the case
    // then comp needs to put a pice is the 3rd box to bolck or win.
    // INPUT: row, col - is where the piece is to be placed (set by func)
    //        boardVal - the piece to check for two in a row
    // OUTPUT: true if valid loc for piece found
    bool checkOneMove(int& row, int& col, int boardVal);
    
    // This checks various configurations for player to win in two moves.
    // INPUT: row, col - is where the piece is to be placed (set by func)
    // OUTPUT: true if valid loc for piece found
    bool checkBlockIn2(int& row, int& col);
    
    // This function returns the 1st blank space found.
    // INPUT: row, col - is the loc of 1st blank space found (set by func)
    // OUTPUT: true if valid loc for piece found
    bool moveToNextAvailableSpace(int& row, int& col);

    // Initialize the winIn2 and locForMove matrices with inital vals.
    void initWinIn2Matrix();
    
    // This function will rotate the board member matrix by 90 deg CCW
    void rotateBoard();
    
    // This function will compare the board member matrix to the winIn2
    // matrix at given location.
    // INPUT: boardPossition - the location in winIn2 matrix to which board
    //                         matrix will be compared to.
    // OUTPUT: true if valic loc for piece found
    bool compare(int boardPossition);
};

#endif
