/*******************************************************
 * TIC TAC TOE Game                                    *
 * by Parthipan Siva                                   *
 *                                                     *
 * Created as demonstration of OOP for                 *
 * SD121 course.                                       *
 *                                                     *
 *  Nov 2005   created                 - Parthipan     *
 *  Nov 2006   added more comments     - Parthipan     *
 *             added definitions for                   *
 *                  UI_H_VS_C                          *
 *                  UI_H_VS_H                          *
 *                  UI_AGAIN_SAME_PLAYERS              *
 *                  UI_AGAIN_NEW_PLAYERS               *
 *                  UI_QUIT                            *
 *                                                     *
 *******************************************************/

#include "Game.h"

Game::Game()
{
     // initialize member variables
     GameStatus = G_INPROGRESS;   // game is in progress
     numGames = 0;                // 0 games played so far
     isPlayerVsComputer = false;  // assume this is false, will be set later
     Player1 = NULL;              // no players created yet
     Player2 = NULL;
     playersMove = 1;             // player one will move 1st
}

Game::~Game()
{
     // free memories created in this class
     delete Player1;
     delete Player2;
     Player1 = NULL;
     Player2 = NULL;
}

void Game::setPlayers()
{
     // Need to determine if it is a Human vs Human or Computer vs 
     // Human game.
     int typeOfGame = getTypeOfGame();
     
     if (typeOfGame == UI_H_VS_C)
     {
           // Human vs Comp game 
           system("cls");
           cout << "Human Player Info: " << endl;
           
           // set Player1 as a new HumanPlayer
           Player1 = new HumanPlayer();
           
           // Get Human Player's info. Allow the player to choose if they will
           // use X or O.
           getPlayerProperties(Player1, true);
           
           // set Player2 as new ComputerPlayer.
           Player2 = new ComputerPlayer();
           
           // Set the computer player's pice as the opposite of Player1
           Player2->set_piece((Player1->get_piece()==Player1->get_X_VAL())
                               ? Player1->get_O_VAL():Player1->get_X_VAL());
           
           // Set that this is a player vs computer game.                    
           isPlayerVsComputer = true;
     }
     else
     {
           // Human vs Human game
           system("cls");
           cout << "Player 1 Info: " << endl;
           
           // set Player1 as a new HumanPlayer
           Player1 = new HumanPlayer();
           
           // Get Player1's info
           getPlayerProperties(Player1, true);
           cout << endl << "Player 2 Info: " << endl;
           
           // set Player2 as new HumanPlayer
           Player2 = new HumanPlayer();
           
           // Get Player2's info, player 2 doesn't get to choose X or O
           getPlayerProperties(Player2, false);
           
           // This is not a human vs computer game.
           isPlayerVsComputer = false;
     }
}

int Game::getTypeOfGame()
{
     int typeOfGame = 0;
     while (typeOfGame!=UI_H_VS_C && typeOfGame!=UI_H_VS_H)
     {
           // Ask the user for what type of game
           cout << "Please select one of the bellow options:" << endl;
           cout << UI_H_VS_C << " Human vs Computer." << endl;
           cout << UI_H_VS_H << " Human vs Human." << endl;
     
           cin >> typeOfGame;
           
           // Keep asking till a valid answer
           if (typeOfGame!=UI_H_VS_C && typeOfGame!=UI_H_VS_H)
           {
               cin.clear();
               cin.ignore(1000,'\n');
               system("cls");
               cout << "Incorrect selection." << endl;
           }
     }
     
     // Ensure that the input buffer is clear
     cin.clear();
     cin.ignore(1000,'\n');
     
     return typeOfGame;
}

void Game::getPlayerProperties(Player * CurrentPlayer, bool getPieceFromUser)
{
     // Ask user for name
     string PlayerName;
     cout << "Enter name: ";
     
     // get player name
     getline(cin, PlayerName, '\n');
     
     // if needed ask user for playing pice (X or O)
     if (getPieceFromUser)
     {
        char piece='c';
        while (piece!='X' && piece!='x' && piece!='O' && piece!='o')
        {
           cout << "Enter piece (X or O): ";
           cin >> piece;
           cin.clear();
           cin.ignore(1000,'\n');
        }
        // set the player piece in player class
        CurrentPlayer->set_piece( (piece=='X'||piece=='x')
                     ? CurrentPlayer->get_X_VAL():CurrentPlayer->get_O_VAL());
     }
     else
     {
         // if not needed set opposite of player 1
         // only player two will get a chance to not enter piece
         CurrentPlayer->set_piece((Player1->get_piece()==Player1->get_X_VAL())
                               ? Player1->get_O_VAL():Player1->get_X_VAL());
     }
     
     // set player name in class
     CurrentPlayer->set_name(PlayerName);
}

void Game::resetGameAndPlayer()
{
     // If the game is reset then all member variables must be reset.
     GameStatus = G_INPROGRESS;
     numGames = 0;
     playersMove = 1;
     isPlayerVsComputer = false;
     Player1->resetPlayer();      // reset players by calling the reset function
     Player2->resetPlayer();
     board.resetBoard();          // reset game board by calling the reset fn
}
    
int Game::playAGame()
{
     // Make sure there are valid players before starting the game
     if (Player1 != NULL && Player2 != NULL)
     {
        // Keep playing till game is over
        while (GameStatus == G_INPROGRESS)
        {
           system("cls");
           if (playersMove == 1)
           {
               // Player1's move (Player1 is always a Human)
               // That is there is no computer vs computer game.
               playersMove = 2; // next time player 2 will move
               board.display_board();  // display the game bord
               // ask the user for a move
               cout << endl;
               cout << "Player 1 (" << ((Player1->get_piece()==
                   Player1->get_X_VAL()) ? "X": "O") << ") Move:" << endl;
               cout << endl;
               // Cast the Player pointer as a HumanPlayer then call the
               // playAMove function to play a move.
               ((HumanPlayer*)Player1)->playAMove(board);
           }
           else
           {
               // Player2's move (Player2 can be either a computer player
               // or human player)
               playersMove = 1;  // Player 1 will move next time
               if (isPlayerVsComputer == false)
               {
                   // 2nd player is also human so ask the player to put a piece
                   // on the bord.
                   board.display_board();
                   cout << endl;
                   cout << "Player 1 (" << ((Player2->get_piece()==
                       Player1->get_X_VAL()) ? "X": "O") << ") Move:" << endl;
                   cout << endl;
                   // Cast the Player pointer as a HumanPlayer
                   ((HumanPlayer*)Player2)->playAMove(board);
               }
               else
               {
                   // The Player2 is the computer so have the computer play a
                   // move ie place a piece.
                   
                   // Cast the Player pointer as a ComputerPlayer
                   ((ComputerPlayer*)Player2)->playAMove(board);
               }
           }
           // Need to see the game status after each move.
           setGameStatus();
           // if the game is not in progress, ie it has ended, then run the
           // end game routine.
           if (GameStatus != G_INPROGRESS)
              endGame();
        }
        return 1; // on successfull game play return 1.
     }
     else
     {
         return -1; // if players are NULL then return -1.
     }
}

void Game::endGame()
{
     // Display what happend in the game, ie who won or if the game was quit.
     system("cls");
     if (GameStatus == G_TIE)
     {
          // tie game
          board.display_board();
          cout << endl;
          cout << "Tie Game" << endl;
     }
     else if (GameStatus == G_P1_WON)
     {
          // Player 1 won
          board.display_board();
          cout << endl;
          cout << "Player 1 '" << Player1->get_name() << "' won" << endl;
     }
     else if (GameStatus == G_P2_WON)
     {
          // Player 2 won
          board.display_board();
          cout << endl;
          cout << "Player 2 '" << Player2->get_name() << "' won" << endl;
     }
     else if (GameStatus == G_QUIT)
     {
          // The game was quit by player
          board.display_board();
          cout << endl;
          cout << "Early end of game." << endl;
     }
     cout << endl;
     cout << endl;
     
     // Print the player status after the game has ended.
     printPlayerStats();
}

void Game::setGameStatus()
{
     // Based on the status of the game board we can determine the status of
     // the game.
     GameBoardStatus boardStatus = board.getBoardStatus();
     if (boardStatus == INPROGRESS)
     {
          // game not over
          GameStatus = G_INPROGRESS;
     }
     else if (boardStatus == X_WIN)
     {
          // Player with X wins
          numGames++;  // increment the number of games played
          if (Player1->get_piece() == Player1->get_X_VAL())
          {
               // Player 1 wins
               GameStatus = G_P1_WON;
               Player1->increment_win(); // increment the number of wins for
                                         // player 1
          }
          else
          {
               // Player 2 wins
               GameStatus = G_P2_WON;
               Player2->increment_win(); // increment the number of wins for
                                         // player 2
          }
     }
     else if (boardStatus == O_WIN)
     {
          // Player with O wins
          numGames++;  // increment the number of games played
          if (Player1->get_piece() == Player1->get_O_VAL())
          {
               // Player 1 wins
               GameStatus = G_P1_WON;
               Player1->increment_win(); // increment the number of wins for
                                         // player 1
          }
          else
          {
               // player 2 wins
               GameStatus = G_P2_WON;
               Player2->increment_win(); // increment the number of wins for 
                                         // player 2
          }
     }
     else if (boardStatus == TIE)
     {
          // Tie game
          numGames++;  // increment the number of games played
          GameStatus = G_TIE;
     }
     else if (boardStatus == QUIT)
     {
          // Player quit before game ended. In this case don't count this as
          // game since it finised before a win or a tie.
          GameStatus = G_QUIT;
     }
}

void Game::printPlayerStats()
{
     // Ensure players are not NULL before printing status.
     if (Player1!=NULL && Player2!=NULL)
     {
        cout << "Player 1 Stats:  " << endl;
        Player1->displayPlayerStat();
        cout << "Number of Games: " << numGames << endl;
     
        cout << endl;
     
        cout << "Player 2 Stats:  " << endl;
        Player2->displayPlayerStat();
        cout << "Number of Games: " << numGames << endl;
     
        cout << endl;
     }
}

int Game::playAgain()
{
     int userChoice = 0;
     
     // Ask the user if they want to play again
     while (userChoice < UI_AGAIN_SAME_PLAYERS || userChoice > UI_QUIT)
     {
          cout << "Please select one of the following: " << endl;
          cout << UI_AGAIN_SAME_PLAYERS << " Play another game with same players. "
               << endl;
          cout << UI_AGAIN_NEW_PLAYERS << " Play another game with new players. "
               << endl;
          cout << UI_QUIT << " Quit. " << endl;
          cin >> userChoice;
          cin.clear();
          cin.ignore(1000,'\n');
     }
     
     if (userChoice == UI_AGAIN_SAME_PLAYERS)
          resetGame();
     else if (userChoice == UI_AGAIN_NEW_PLAYERS)
          resetGameAndPlayer();
     
     // return what the user has chosen
     return userChoice;
}

void Game::resetGame()
{
     // When a game is reset some member variables must be reset.
     GameStatus = G_INPROGRESS;
     playersMove = 1;
     board.resetBoard();
}
