/*******************************************************
 * TIC TAC TOE Game                                    *
 * by Parthipan Siva                                   *
 *                                                     *
 * Created as demonstration of OOP for                 *
 * SD121 course.                                       *
 *                                                     *
 *  Nov 2005   created                 - Parthipan     *
 *  Nov 2006   added more comments     - Parthipan     *
 *                                                     *
 *******************************************************/
 
#include "GameBoard.h"

GameBoard::GameBoard()
{
     // reset the bord
     resetBoard();
     // set status as inprogress
     BoardStatus = INPROGRESS;
}

void GameBoard::resetBoard()
{
     // Initialize the board to blanks
     for (int r = 0; r < 3; r++)
     {
         for (int c = 0; c < 3; c++)
         {
             i_GameBoard[r][c] = BLANK_VAL;
         }
     }
}

void GameBoard::display_board() const
{
     // Clear the screen
     system("cls");
     
     // output board to screen
     cout << " | 1 | 2 | 3 |" << endl;
     cout << " |-----------|" << endl;
     for (int j = 0; j < 3; j++)
     {
         cout << j+1;
         cout << "| ";
         for (int i = 0; i < 3; i++)
         {
             if (i_GameBoard[j][i] == X_VAL)
                cout << "x";
             else if (i_GameBoard[j][i] == O_VAL)
                cout << "o";
             else
                cout << " ";
             cout << " | ";
         }
         cout << endl << " |-----------|" << endl;
     }
}

int GameBoard::o_move(int row, int col)
{
     // ensure that row and col are valid values. That is that they are between
     // 0 and 2 inclusive and that there is a blank spot on the bord at that
     // point.
     if (row >-1 && row < 3 && col >-1 && col < 3 && 
          i_GameBoard[row][col]==BLANK_VAL)
     {
         // place the game piece
         i_GameBoard[row][col] = O_VAL;
         // get the bord status after piece is placed
         boardStatus();
         return 1;  // return 1 for success
     }
     else
     {
         boardStatus();
         return -1;  // return -1 for error
     }
}

int GameBoard::x_move(int row, int col)
{
     // ensure that row and col are valid values. That is that they are between
     // 0 and 2 inclusive and that there is a blank spot on the bord at that
     // point.
     if (row >-1 && row < 3 && col >-1 && col < 3 && 
          i_GameBoard[row][col]==BLANK_VAL)
     {
         // place the game piece
         i_GameBoard[row][col] = X_VAL;
         // get the board status after the piece is placed
         boardStatus();
         return 1;  // return 1 for success
     }
     else
     {
         boardStatus();
         return -1; // return -1 for error
     }
}

void GameBoard::boardStatus()
{
     // Get the bord status
     if (isThreeInRow(X_VAL))
        // X wins if there are 3 x in a row
        BoardStatus = X_WIN;
     else if (isThreeInRow(O_VAL))
        // O wins if there are 3 o in a row
        BoardStatus = O_WIN;
     else if (isTie())
        // tie game if there is no move left and x and o have not won
        BoardStatus = TIE;
     else
        // if x not won and o not won and not tie then game is in progress
        BoardStatus = INPROGRESS;
}

bool GameBoard::isThreeInRow(int val) const
{
     bool hasThreeInRow = false;
     
     // check rows
     for (int i = 0; i < 3 && hasThreeInRow == false; i++)
     {
         hasThreeInRow = true;
         for (int j = 0; j < 3 && hasThreeInRow == true; j++)
         {
             if (i_GameBoard[i][j] != val)
                 hasThreeInRow = false;
         }
     }
     
     // check cols
     for (int i = 0; i < 3 && hasThreeInRow == false; i++)
     {
         hasThreeInRow = true;
         for (int j = 0; j < 3 && hasThreeInRow == true; j++)
         {
             if (i_GameBoard[j][i] != val)
                 hasThreeInRow = false;
         }
     }
     
     // check diags
     if (hasThreeInRow == false)
     {
         if (i_GameBoard[0][0] == val &&
             i_GameBoard[1][1] == val &&
             i_GameBoard[2][2] == val)
             hasThreeInRow = true;
         else if (
             i_GameBoard[0][2] == val &&
             i_GameBoard[1][1] == val &&
             i_GameBoard[2][0] == val)
             hasThreeInRow = true;
     }
     
     return hasThreeInRow;
}

bool GameBoard::isTie() const
{
     bool isTie = true;
     for (int i = 0; i < 3; i++)
     {
         for (int j = 0; j < 3; j++)
         {
             // ensure that there is no blank value for tie game
             if (i_GameBoard[i][j] == BLANK_VAL)
                 isTie = false;
         }
     }
     return isTie;
}

void GameBoard::getBoard(int board[][3]) const
{
     // return bord by copying board matrix to given variable
     for (int r = 0; r < 3; r++)
         for (int c = 0; c < 3; c++)
             board[r][c] = i_GameBoard[r][c];
}

void GameBoard::quitGame()
{
     // quit game
     BoardStatus = QUIT;
}
