/*******************************************************
 * TIC TAC TOE Game                                    *
 * by Parthipan Siva                                   *
 *                                                     *
 * Created as demonstration of OOP for                 *
 * SD121 course.                                       *
 *                                                     *
 *  Nov 2005   created                 - Parthipan     *
 *  Nov 2006   added more comments     - Parthipan     *
 *                                                     *
 *******************************************************/
 
 /*
 This is a Player class that is the core class for any
 computer or human players. It will keep track of player
 name and stats.
 */
 
#ifndef PLAYER_H
#define PLAYER_H

#include <iostream>
#include <stdlib.h>
#include <string>
using namespace std;

class Player
{
public:

    Player();
    Player(string playerName);
    
    // This function will set whether player is using X or O.
    // INPUTS: 1 - X, 2 - O
    // SIDEEFFECT: sets the member veriable piece
    void set_piece(int piece_val);
    
    // This function will return if player is returning X or O.
    // OUTPUT: 1 - X, 2 - O
    int get_piece() const;
    
    // This function will set the player's name.
    // INPUTS: player name
    // SIDEEFECT: sets the member variable name
    void set_name(string playerName);
    
    // This function will return player's name.
    // OUTPUT: player's name
    string get_name() const;
    
    // This function will return number of games won.
    // OUTPUT: number of games won
    int get_num_wins() const;
    
    // This function will increment the number of games won by 1.
    // SIDEEFFECT: increment member variable num_wins by 1.
    void increment_win();
    
    // This will reset player info
    // SIDEEFFECT: set piece, num_games, num_wins to zero and name to ""
    void resetPlayer();
    
    // This function will display the stats for this player
    void displayPlayerStat();
    
    // This function will return the const X_VAL
    int get_X_VAL();
    
    // This function will return the const O_VAL
    int get_O_VAL();
    
    // This function will return the const BLANK_VAL
    int get_BLANK_VAL();

private:

    string name;  // name of player
    int piece;    // the player piece ie X or O used by the player
    int num_wins; // number of games won
    
    // Constants used to keep track of X, O and Blanks
    static const int X_VAL = 1;
    static const int O_VAL = 2;
    static const int BLANK_VAL = -1;
};

#endif
